;;
;; CPMLIB.MAC
;;
;; Macro Bibliothek fuer CP/M 2.2 & 3.x mit M80 aus dem
;; SYBEX Buch "Programmieren mit CP/M" von Alan R. Miller
;;
;; Erweitert und modifiziert von AK
;; 09.01.1987 - 22.06.1987
;;
;; Rev.		30.10.1987
;;		27.11.1988	BIOSC
;;	1.04	04.04.1988	Steuerzeichen
;;
;;	1.05	30.12.2005	HLDEC
;;	1.06	06.05.2021	READB mit Trennzeichenauswertung (CPM2LIB)
;;				BIOSC2 & BIOSC3 je nach CP/M Version
;;	1.07	17.12.2021	READB ohne Trennzeichenauswertung
;;				IDETOOL <LBA> funktioniert sonst nicht
;;
BLANK	EQU	' '
CLS	EQU	1AH
COMMA	EQU	','
POINT	EQU	'.'
;
BDOS	EQU	5
BOOT	EQU	0
DBUFF	EQU	80H
FCB1	EQU	5CH
FCB2	EQU	6CH
IOBYTE	EQU	3
;
NULL	EQU	0		; NUL = M80 Befehl
SOH	EQU	1
STX	EQU	2
ETX	EQU	3
EOT	EQU	4
ENQ	EQU	5
ACK	EQU	6
BEL	EQU	7		; BELL = M80 Befehl
BS	EQU	8
HT	EQU	9
LF	EQU	0AH
VT	EQU	0BH
FF	EQU	0CH
CR	EQU	0DH
SO	EQU	0EH
SI	EQU	0FH
DLE	EQU	10H
DC1	EQU	11H
DC2	EQU	12H
DC3	EQU	13H
DC4	EQU	14H
NAK	EQU	15H
SYN	EQU	16H
ETB	EQU	17H
CAN	EQU	18H
EM	EQU	19H
SUB	EQU	1AH
EOF	EQU	1AH
ESC	EQU	1BH
FS	EQU	1CH
GS	EQU	1DH
RS	EQU	1EH
US	EQU	1FH
;;
;; Macros 				Flags
;; ==========================================
;;
;; ABORT	CHAR			CIFLAG,COFLAG
;; AMBIG	OLD,NEW			-
;; BIOSC2	FUNC			BIFLAG
;; BIOSC3	FUNC			BIFLAG
;; CLOSE	POINTR			CLFLAG,COFLAG,CRFLAG
;;					PRFLAG,OPFLAG
;;					DEFLAG,CIFLAG,UNFLAG
;;					RNFLAG,S2FLAG
;; CLREOL				EOLFLG
;; COMPAR	FIRST,SECOND,BYTES	CMFLAG
;; COMPRA	FIRST,SECOND,BYTES	CMFLAG
;; CRLF		-			-
;; DELETE	POINTR,WHERE		DEFLAG,CIFLAG,COFLAG
;;					PRFLAG,UNFLAG
;; DOSVER	-			-
;; ENDE		-			-
;; ENTER	-			-
;; ERRORM	TEXT,WHERE		COFLAG,CRFLAG,PRFLAG
;; EXIT		SPACE?		  	-
;; FILL		ADDR,BYTES,CHAR		-
;; GFNAME	FCB			FNFLAG,REFLAG
;;					COFLAG,CRFLAG,PRFLAG
;; GOTOXY	X,Y			GOFLAG
;; HEXHL	POINTR			HXFLAG,RCFALG
;; HLDEC	SBC,PCHAR		DEFLAG
;; LCHAR	PAR			LOFLAG
;; LDFILE	FCB,POINTR,STAR		COFLAG,DMFLAG,RDFLAG
;; LDSYSF	FCB,POINTR,STAR		- " -
;; MAKE		POINTR			MKFLAG,COFLAG,CRLFAG
;;					PRFLAG
;; MOVE		FROM,TO,BYTES		-
;; OPEN		POINTR,WHERE		OPFLAG,COFLAG,PRFLAG
;; OUTHEX	REG			CXFLAG,COFLAG
;; OUTHL				CXFLAG,COFLAG
;; PCHAR	PAR			COFLAG
;; PFNAME	FCB			COFLAG,PRFLAG
;; PRINT	TEXT,BYTES		PRFLAG,COFLAG
;; PROTEC	POINTR			-
;; READB	BUFFR			RCFLAG
;; READCH	REG			CIFLAG,COFLAG
;; READS	POINTR,STAR		RDFLAG,COFLAG
;; RENAME	POINTR			RNFLAG,COFLAG,PRFLAG
;;					CRFLAG
;; SETDMA	POINTR			DMFLAG
;; SETUP2	-			S2FLAG,CIFLAG,COFLAG
;;					CRFLAG,CMFLAG,DEFLAG
;;					MKFLAG,OPFLAG
;;					PRFLAG,UNFLAG
;; SYSF		FUNC,AE			-
;; UCASE	REG			-
;; UNPROT	POINTR			UNFLAG
;; UPPER	REG			-
;; VERSN	TEXT			-
;; WAIT		TIME			WAFLAG
;; WRFILE	FCB,POINTR		COFLAG,CRFLAG,DMFLAG
;;					WRFLAG
;; WRITES	POINTR,STAR		WRFLAG,COFLAG,DMFLAG
;;
;
;
;
ABORT	 MACRO	CHAR
;; 18.08.1986
;; Inline Macro zum Abruch eines
;; Programms.
;;
;; Anwendung:	ABORT
;;		ABORT	ESC
;;
;; Extern:	READCH
;;
	LOCAL AROUND
	PUSH	H
	PUSH	D
	PUSH	B
	MVI	C,11
	CALL	BDOS
	POP	B
	POP	D
	POP	H
	RRC
	JNC	AROUND
	READCH
	 IF	NUL CHAR
	JMP	DONE
	 ELSE
	CPI	CHAR
	JZ	DONE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
AMBIG	 MACRO	OLD,NEW
;; 16.08.1986
;; Inline Macro wandelt einen nicht
;; eindeutigen Dateinamen bei FCB OLD
;; in einen eindeutigen nach FCB NEW.
;;
;; Anwendung:	AMBIG 	FCB1,FCB2
;;
	PUSH	H
	PUSH	D
	PUSH	B
	LXI	H,NEW+1
	LXI	D,OLD+1
	MVI	C,11
;
AMB2?:	MVI	A,'?'
	CMP	M
	JNZ	AMB3?
	LDAX	D
	MOV	M,A
;
AMB3?:	INX	H
	INX	D
	DCR	C
	JNZ	AMB2?
	POP	B
	POP	D
	POP	H
;;
	 ENDM
;
;
;
BIOSC2	 MACRO	FUNC
;; Macro zum generieren von CP/M2 BIOS Calls.
;; FUNC ist die entsprechende BIOS-Funktion.
;;
;; Anwendung:	BIOSC2
;;		BIOSC2	FUNC
;; 
;; Extern:	BOOT+1
;;
	LOCAL AROUND
	 IF	NOT NUL FUNC
	MVI	A,FUNC
	 ENDIF
;
	CALL	BIOSC2?
	 IF	NOT BIFLAG
	JMP	AROUND
;
BIOSC2?:
	PUSH	D
	DCR	A
	MOV	D,A
	# SLA A
	ADD	D
	LHLD	BOOT+1		; Wboot-Vektor
	MVI	D,0
	MOV	E,A
	DAD	D
	POP	D
	PCHL
;
BIFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
BIOSC3	 MACRO	FUNC
;; Macro zum generieren von CP/M3 BIOS-Calls.
;; FUNC ist die entsprechende BIOS-Funktion.
;;
;; Anwendung:	BIOSC3
;;		BIOSC3	FUNC
;; 
	LOCAL AROUND

	 IF	NOT NUL FUNC
	MVI	A,FUNC
	 ENDIF
;
	STA	BFUNC
	CALL	BIOSC3?
	JMP	AROUND

	 IF	NOT BIFLAG
;
BIOSC3?:
	# LD (BCREG),BC
	# LD (DEREG),DE
	SHLD	HLREG
	LXI	D,BIOSPB
	MVI	C,32H		; bios funktion #50
	CALL	BDOS
	RET
;
BIOSPB	EQU	$

BFUNC:	DB	0
AREG:	DB	0
BCREG:	DW	0
DEREG:	DW	0
HLREG:	DW	0

BIFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM

;
;
;
CLOSE	 MACRO	POINTR
;; 19.08.1986
;; Inline Macro zum schliessen einer neuen
;; Datei. POINTR zeigt auf Dateisteuerblock.
;;
;; Anwendung:	CLOSE	DFCB
;;
;; Extern:	SYSF,ERRORM,OPEN,PRINT,MOVE,DELETE,RENAME
;;
	LOCAL AROUND,CLOS3?
	 IF	NOT NUL POINTR
	LXI	D,POINTR
	 ENDIF
	CALL	CLOS2?
	INR	A
	 IF	NOT S2FLAG
	JNZ	AROUND
	 ELSE
	JZ	CLOS3?
	LDA	DUPL
	ORA	A
	JZ	AROUND
	MOVE 'BAK',FCB1+10H+9
	MOVE FCB1+9,DFCB+10H+9,3
	MOVE FCB1,FCB1+10H,9
	MOVE DFCB,DFCB+10H,9
	DELETE FCB1+10H
	RENAME FCB1
	RENAME DFCB
	MOVE 'BAK',FCB1+9
	OPEN FCB1
	JMP	AROUND
;
	 ENDIF
	 IF	NOT CLFLAG
;
CLOS3?:	GOTOXY 3,22
	CLREOL
	PRINT <BEL,'Kann File nicht schliessen!'>
	STC
	JMP	AROUND
;
CLOS2?:	SYSF 16
;
CLFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
CLREOL	 MACRO	
;; 09.01.1987
;; Clear end of Line
;;
	LOCAL AROUND
	CALL	EOL?
	 IF	NOT EOLFLG
	JMP	AROUND
;
EOL?:	PCHAR ESC
	PCHAR 'T'
	RET
;
EOLFLG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
COMPAR	 MACRO	FIRST,SECOND,BYTES
;; 16.08.1986
;; Inline Macro zum Vergleich zweier
;; Bereiche. Das Z-Flag ist gesetzt,
;; wenn beide Bereiche gleich sind.
;; Der erste Parameter darf eine
;; Textkonstante sein.
;;
;; Anwendung:	COMPAR FCB1,FCB2,12
;;		COMPAR '???',FCB1+9
;;		COMPAR ,,5
;;
	LOCAL MESG,AROUND
	PUSH	H
	PUSH	D
	PUSH	B
	 IF	NUL BYTES
	LXI	H,MESG
	MVI	C,AROUND-MESG
	 ELSE
	 IF	NOT NUL FIRST
	LXI	H,FIRST
	 ENDIF
	 IF	NOT NUL BYTES
	MVI	C,BYTES
	 ENDIF
	 ENDIF
	 IF	NOT NUL SECOND
	LXI	D,SECOND
	 ENDIF
	CALL	COMP2?
	POP	B
	POP	D
	POP	H
	 IF	NOT CMFLAG OR NUL BYTES
	JMP	AROUND
	 ENDIF
	 IF	NOT CMFLAG
;
COMP2?:	LDAX	D
	CMP	M
	RNZ
	INX	H
	INX	D
	DCR	C
	JNZ	COMP2?
	RET
;
CMFLAG	SET	TRUE
	 ENDIF
	 IF	NUL BYTES
MESG:	DB	FIRST
	 ENDIF
AROUND:
;;
	 ENDM
;
;
;
COMPRA	 MACRO	FIRST,SECOND,BYTES
;; 16.08.1986
;; ASCII Version (Bit 7 wird geloescht).
;; Inline Macro zum Vergleich zweier
;; Bereiche. Das Z-Flag ist gesetzt,
;; wenn beide Bereiche gleich sind.
;; Der erste Parameter darf eine
;; Textkonstante sein.
;;
;; Anwendung:	COMPRA FCB1,FCB2,12
;;		COMPRA '???',FCB1+9
;;		COMPRA ,,5
;;
	LOCAL MESG,AROUND
	PUSH	H
	PUSH	D
	PUSH	B
	 IF	NUL BYTES
	LXI	H,MESG
	MVI	C,AROUND-MESG
	 ELSE
	 IF	NOT NUL FIRST
	LXI	H,FIRST
	 ENDIF
	 IF	NOT NUL C
	MVI	C,BYTES
	 ENDIF
	 ENDIF
	 IF	NOT NUL SECOND
	LXI	D,SECOND
	 ENDIF
	CALL	COMP2?
	POP	B
	POP	D
	POP	H
	 IF	NOT CMFLAG OR NUL BYTES
	JMP	AROUND
	 ENDIF
	 IF	NOT CMFLAG
COMP2?:	LDAX	D
	ANI	7FH
	PUSH	B
	MOV	C,A
	MOV	A,M
	ANI	7FH
	CMP	C
	POP	B
	RNZ
	INX	H
	INX	D
	DCR	C
	JNZ	COMP2?
	RET
;
CMFLAG	SET	TRUE
	 ENDIF
	 IF	NUL BYTES
MESG:	DB	FIRST
	 ENDIF
AROUND:
;;
	 ENDM
;
;
;
DOSVER	 MACRO	
;; 16.08.1986
;; Inline Macro zum bestimmen der DOS-Version.
;; Reg A enthaelt Version in BCD mal 10.
;;
;; Extern:	BDOS
;;
	PUSH	H
	PUSH	D
	PUSH	B
	MVI	C,12
	CALL	BDOS
	MOV	A,L
	POP	B
	POP	D
	POP	H
;;
	 ENDM
;
;
;
CRLF	 MACRO	
;; 16.08.1986
;; Inline Macro fuer die Ausgabe
;; einer neuen Zeile.
;;
;; Extern:	PCHAR
;;
	LOCAL AROUND
	CALL	CRLF2?
	 IF	NOT CRFLAG
	JMP	AROUND
;
CRLF2?:	PUSH	PSW
	PCHAR CR
	PCHAR LF
	POP	PSW
	RET
CRFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
DELETE	 MACRO	POINTR,WHERE
;; 19.08.1986
;; Inline Macro zum loeschen einer Datei.
;; Wenn die Datei schreibgeschuetzt ist,
;; weiter bei WHERE oder DONE.
;;
;; Extern:	SYSF,UNPROT,READCH,PFNAME,PRINT,UCASE,CRLF
;;
	LOCAL AROUND,DEL1?
	LXI	D,POINTR
	LDA	POINTR+9
	ANI	80H
	JZ	DEL1?
	CALL	WPERR?
	 IF	NOT NUL WHERE
	JNZ	WHERE
	 ELSE
	JNZ	DONE
	 ENDIF
	UNPROT POINTR
;
DEL1?:	CALL	DEL2?
	JMP	AROUND
;
	 IF	NOT DEFLAG
DEL2?:	SYSF 19
;
WPERR?:	GOTOXY 3,22
	CLREOL
	PRINT <BEL,'File ist schreibgeschuetzt, loeschen (J/N) : '>
	READCH
	UCASE
	CPI	'J'
	RET
;
DEFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
ENDE	 MACRO	
;; 24.08.1986
;; Inline Macro zum aufuellen
;; des Blocks mit Null-Zeichen.
;; Nur bei M80 noetig.
;;
ENDE?	EQU	$
;
ANZAHL? EQU	ENDE? MOD 256
;
	DS	256-ANZAHL?,0
;;
	 ENDM
;
;
;
ENTER	 MACRO	
;; 16.08.1986	Neu 25.08.1986
;; Inline Macro zum retten des
;; uebergebenen Stack-Pointers.
;;
	# LD (OLDSTK),SP	; retten
	LXI	SP,STACK	; User-Stack
;;
	 ENDM
;
;
;
ERRORM	 MACRO	TEXT,WHERE
;; 18.08.1986
;; Macro fuer die Ausgabe einer
;; Fehlermeldung. Optionaler zweiter
;; Parameter ist Sprungadresse.
;; Fehlt der zweite Parameter dann
;; Warmstart.
;;
;; Anwendung:	ERRORM	'Meldung'
;;		ERRORM	'Meldung',DONE
;;
;; Extern:	PRINT, CRLF
;;
	CRLF
	PRINT <TEXT>
	 IF	NUL WHERE
	JMP	BOOT
	 ELSE
	JMP	WHERE
	 ENDIF
;;
	 ENDM
;
;
;
EXIT	 MACRO	WHERE?,SPACE?
;; 16.08.1986	Neu 25.08.1986
;; Inline Macro zum Retten des
;; uebergebenen Stack-Pointers
;; und Sprung zu Adresse Where?
;; Fehlt WHERE?, wird ein RET-
;; Befehl ausgefuehrt. SPACE
;; definiert den Stackbereich (34).
;;
;; Anwendung:	EXIT
;;		EXIT,BOOT
;;		EXIT,BOOT,50
;;
	# LD SP,(OLDSTK)
	 IF	NUL WHERE?
	RET
	 ELSE
	JMP	WHERE?
	 ENDIF
;
OLDSTK:	DW	0
	 IF	NUL SPACE?
	DS	34,0
	 ELSE
	DS	SPACE?,0
	 ENDIF
STACK:	
;;
	 ENDM
;
;
;
FILL	 MACRO	ADDR,BYTES,CHAR
;; 29.08.1986
;; Inline Macro zum fuellen des Speichers
;; von ADDR mal BYTES mit dem Zeichen CHAR.
;;
;; Anwendung:	FILLD FCB+1,BLANK,8
;;		FILLD FCB+9,'?',3
;;
	PUSH	H
	PUSH	D
	PUSH	B
	 IF	NOT NUL ADDR
	LXI	H,ADDR
	LXI	D,ADDR+1
	 ELSE
	PUSH	H
	POP	D
	INX	D
	 ENDIF
	LXI	B,BYTES-1
	MVI	A,CHAR
	MOV	M,A
	# LDIR
	POP	B
	POP	D
	POP	H
;;
	 ENDM
;
;
;
GFNAME	 MACRO	FCB
;; 18.08.1986
;; Inline Macro fuer die Eingabe eines
;; Dateinamens von der Konsole.
;;
;; Extern:	READB, FILL, UCASE
;;
	LOCAL AROUND,PNAME,ENAME,EXTEN,GCHAR,GNAM2
	PUSH	H
	PUSH	D
	PUSH	B
	LXI	H,FCB
	SHLD	FCBS?
	CALL	GNAM?
	POP	B
	POP	D
	POP	H
	 IF	NOT FNFLAG
	JMP	AROUND
;
FCBS?:	DS	2,0
;
GNAM?:	CRLF
;
GNAM2:	GOTOXY 3,22
	CLREOL
	PRINT 'Filename : '
	LHLD	FCBS?
	XRA	A
	MOV	M,A
	INX	H
	FILL ,11,BLANK
	XCHG
	READB
	CALL	GETCH
	JC	GNAM2
	CPI	BLANK
	JZ	GNAM2
	UCASE
	STAX	D
	CALL	GETCH
	RC
	CPI	BLANK
	RZ
	MVI	B,7
	UCASE
	CPI	POINT
	JZ	ENAME
	CPI	':'
	JNZ	PNAME
	LDAX	D
	SUI	'A'-1
	STAX	D
	CALL	GETCH
	JC	GNAM2
	UCASE
	INR	B
	DCX	D
;
PNAME:	INX	D
	STAX	D
	CALL	GETCH
	RC
	CPI	BLANK
	RZ
	UCASE
	CPI	POINT
	JZ	ENAME
	DCR	B
	JNZ	PNAME
	JMP	GNAM2
;
ENAME:	LHLD	FCBS?
	LXI	D,9
	DAD	D
	XCHG
	MVI	B,3
;
EXTEN:	CALL	GETCH
	RC
	CPI	BLANK
	RZ
	UCASE
	STAX	D
	INX	D
	DCR	B
	JNZ	EXTEN
	RET
;
FNFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
GOTOXY	 MACRO	X?,Y?
;; 09.01.1987
;; Inline Macro zur direkten
;; Kursorpositionierung.
;;
;; Extern : PRINT
;;
;; Anwendung :	GOTOXY	10,5
;;
	LOCAL AROUND
	PUSH	B
	MVI	A,X?
	MVI	B,Y?
	CALL	INIT?
	POP	B
	 IF	NOT GOFLAG
	JMP	AROUND
;
INIT?:	ADI	31
	STA	INIT$+3
	MOV	A,B
	ADI	31
	STA	INIT$+2
	PRINT INIT$,4
	RET
;
INIT$:	DB	ESC,3DH
	DB	0
	DB	0
;
GOFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
HEXHL	 MACRO	
;; 16.08.1986
;; Inline Macro zum konvertieren von
;; ASCII - HEX Zeichen.
;;
;; Extern:	READB, UCASE
;;
	LOCAL AROUND,RDHL2,NIB?
	CALL	RDHL?
	 IF	NOT HXFLAG
	JMP	AROUND
;
RDHL?:	LXI	H,0
RDHL2:	CALL	GETCH
	CMC
	RNC
	UCASE
	CALL	NIB?
	RC
	DAD	H
	DAD	H
	DAD	H
	DAD	H
	ORA	L
	MOV	L,A
	JMP	RDHL2
;
NIB?:	SUI	'0'
	RC
	CPI	'F'-'0'+1
	CMC
	RC
	CPI	10
	CMC
	RNC
	SUI	'A'-'9'-1
	CPI	10
	RET
;
HXFLAG	SET	TRUE
	 ENDIF
AROUND:
;;
	 ENDM
;
;
;
HLDEC	 MACRO	
;; 30.12.2005
;; Inline Makro fuer die Ausgabe von HL als Dezimalzahl
;;
;; Extern:	SBC, PCHAR
;;
	LOCAL AROUND,SUBTR,SUBT2,NZERO
	CALL	HLDC2?
	 IF	NOT DEFLAG
	JMP	AROUND
;
HLDC2?:
	PUSH	H
	PUSH	D
	PUSH	B
	MVI	B,0
	LXI	D,-10000
	CALL	SUBTR
	LXI	D,-1000
	CALL	SUBTR
	LXI	D,-100
	CALL	SUBTR
	LXI	D,-10
	CALL	SUBTR
	MOV	A,L
	ADI	'0'
	PCHAR
	POP	B
	POP	D
	POP	H
	RET
;;
;; Subtrahiere Zehnerpotenz und zaehle
;;
SUBTR:
	MVI	C,'0'-1
SUBT2:
	INR	C
	DAD	D
	JC	SUBT2
;; einmal zu oft, eine Rueckaddition
;; durch Subtraktion des Zweierkomplements
	# SBC HL,DE
	MOV	A,C
	CPI	'1'
	JNC	NZERO
	MOV	A,B
	ORA	A
	MOV	A,C
	RZ
	PCHAR
	RET
;; Setze Flag fuer Zeichen <> 0
NZERO:
	MVI	B,0FFH
	PCHAR
	RET
DEFLAG	SET	TRUE
	 ENDIF
AROUND:
;;
	 ENDM
;
;
;
LCHAR	 MACRO	PAR
;; 17.08.1986
;; Inline Macro zum ausgeben eines
;; Zeichens auf dem Drucker.
;;
;; Anwendung:	LCHAR '*'
;;		LCHAR CR
;;		LCHAR
;;
;; Extern:	SYSF
;;
	LOCAL AROUND
	 IF	NOT NUL PAR
	MVI	A,PAR
	 ENDIF
	CALL	LCH2?
	 IF	NOT LOFLAG
	JMP	AROUND
;
LCH2?:	SYSF 5,AE
;
LOFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
LDFILE	 MACRO	FCB,POINTR,STAR
;; 24.08.1986
;; Inline Macro zum einlesen einer Datei
;; in den Speicher ab Adresse POINTR.
;;
;; Anwendung:	LDFILE	FCB1,DBUFFP
;;		LDFILE	FCB1,DBUFFP,'*'
;;
;; Extern:	SETDMA,READS
;;
	LOCAL LOAD2?,LOAD3?

LOAD2?:	LHLD	POINTR
	XCHG
	SETDMA
	READS FCB,STAR
	JNZ	LOAD3?
	LHLD	POINTR
	LXI	D,80H
	DAD	D
	SHLD	POINTR
;
; Test ob Datei den CCP ueberschreibt
;
	LDA	7
	SUI	2
	CMP	H
	JNC	LOAD2?
;
LOAD3?:	PUSH	PSW
	SETDMA 80H
	POP	PSW
;;
	 ENDM
;
;
;
LDSYSF	 MACRO	FCB,POINTR,STAR
;; 24.08.1986
;; Inline Macro zum einlesen einer Sys-Datei
;; in den Speicher ab Adresse POINTR.
;;
;; Anwendung:	LDFILE	FCB1,DBUFFP
;;		LDFILE	FCB1,DBUFFP,'*'
;;
;; Extern:	SETDMA,READS
;;
	LOCAL LOAD2?,LOAD3?
LOAD2?:	LHLD	POINTR
	XCHG
	SETDMA
	READS FCB,STAR
	JNZ	LOAD3?
	LHLD	POINTR
	LXI	D,80H
	DAD	D
	SHLD	POINTR
	JMP	LOAD2?
;
LOAD3?:	PUSH	PSW
	SETDMA 80H
	POP	PSW
;;
	 ENDM
;
;
;
MAKE	 MACRO	POINTR
;; 19.08.1986
;; Inline Macro zum erstellen einer
;; neuen Datei. POINTR zeigt auf Datei-
;; Steuerblock. Extend und laufende
;; Satznummer werden geloescht.
;;
;; Extern:	SYSF,ERRORM
;;
	LOCAL AROUND
	LXI	D,POINTR
	XRA	A
	STA	POINTR+12
	STA	POINTR+32
	CALL	MAKE2?
	INR	A
	JNZ	AROUND
	GOTOXY 3,22
	CLREOL
	PRINT <BEL,'Directory ist voll!'>
	STC
	JMP	AROUND
	 IF	NOT MKFLAG
;
MAKE2?:	SYSF 22
;
MKFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
MOVE	 MACRO	FROM,TO,BYTES
;; 16.08.1986
;; Inline Macro zum umspeichern von
;; Bytes oder Text.
;;
	LOCAL AROUND,MESG
	PUSH	H
	PUSH	D
	PUSH	B
	 IF	NOT NUL TO
	LXI	D,TO
	 ENDIF
	 IF	NUL BYTES
	LXI	H,MESG
	LXI	B,AROUND-MESG
	 ELSE
	 IF	NOT NUL FROM
	LXI	H,FROM
	 ENDIF
	LXI	B,BYTES
	 ENDIF
	# LDIR
	POP	B
	POP	D
	POP	H
	JMP	AROUND
;
	 IF	NUL BYTES
MESG:	DB	FROM
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
OPEN	 MACRO	POINTR,WHERE
;; 18.08.1986	Neu 03.09.1986
;; Inline Macro fuer die Eroeffnung
;; einer bestehenden Datei.
;;
;; Extern:	SYSF,ERRORM
;;
	LOCAL AROUND
	LXI	D,POINTR
	XRA	A
	STA	POINTR+12
	STA	POINTR+32
	CALL	OPEN2?
	INR	A
	JNZ	AROUND
	 IF	NOT NUL WHERE
	JMP	WHERE
	 ELSE
	JMP	OPERR?
	 ENDIF
;
	 IF	NOT OPFLAG
OPERR?:	GOTOXY 3,22
	CLREOL
	PRINT <BEL,'Kann File nicht oeffnen!'>
	STC
	JMP	AROUND
;
OPEN2?:	SYSF 15
;
OPFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
OUTHEX	 MACRO	REG
;; 16.08.1986
;; Inline Macro um eine Binaerzahl in
;; REG in zwei HEX-Zeichen umzuwandeln
;; und zu drucken. BYTE wird in A
;; erwartet wenn REG fehlt.
;;
;; Extern:	PCHAR
;;
	LOCAL AROUND,HEX1?,HEX2?
	 IF	NOT NUL REG
	MVI	A,REG
	 ENDIF
	CALL	OUTHX?
	 IF	NOT CXFLAG
	JMP	AROUND
;
OUTHX?:	PUSH	B
	MOV	C,A
	RAR
	RAR
	RAR
	RAR
	CALL	HEX1?
	MOV	A,C
	CALL	HEX1?
	MOV	A,C
	POP	B
	RET
;
HEX1?:	ANI	0FH
	ADI	'0'
	CPI	'9'+1
	JC	HEX2?
	ADI	'A'-'9'-1
;
HEX2?:	PCHAR
	RET
;
CXFLAG	SET	TRUE
	 ENDIF
AROUND:
;;
	 ENDM
;
;
;
OUTHL	 MACRO	
;; 03.01.1987
;; Inline Macro fuer die Anzeige von HL in HEX.
;;
;; Anwendung:	OUTHL
;;
;; Extern:	OUTHEX
;;
	LOCAL OVER
	MOV	A,H
	ORA	A
	JZ	OVER
	OUTHEX H
OVER:	OUTHEX L
;;
	 ENDM
;
;
;
PCHAR	 MACRO	PAR
;; 16.08.1986
;; Inline Macro zum ausgeben eines Zeichens
;; auf der Konsole. Wenn Parameter angegeben,
;; wird in A geladen.
;;
;; Anwendung:	PCHAR
;;		PCHAR	'*'
;;
;; Extern:	SYSF
;;
	LOCAL AROUND
	 IF	NOT NUL PAR
	MVI	A,PAR
	 ENDIF
	CALL	PCH2?
	 IF	NOT COFLAG
	JMP	AROUND
;
PCH2?:	SYSF 2,AE
;
COFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
PFNAME	 MACRO	FCB
;; 19.08.1986
;; Inline Macro zum ausgeben des Datei-
;; namens als FIRST.EXT. FCB ist Datei-
;; steuerblock.
;;
;; Extern:	PCHAR,PRINT
;;
	LOCAL PFNA2?,PFNA3?
	PUSH	H
	PUSH	B
	MVI	B,8
	LXI	H,FCB+1
;
PFNA3?:	MOV	A,M
	PCHAR
	INX	H
	DCR	B
	JNZ	PFNA3?
;
PFNA2?:	POP	B
	POP	H
	PCHAR '.'
	PRINT FCB+9,3
;;
	 ENDM
;
;
;
PRINT	 MACRO	TEXT,BYTES
;; 16.08.1986
;; Inline Macro fuer die Ausgabe eines
;; Textes auf der Konsole. TEXT ist die
;; Adresse des Textes, BYTES die Laenge.
;; TEXT darf in '' eingeschlossen sein,
;; wenn BYTES fehlt.
;;
;; Anwendung:	PRINT	FCB1+1,11
;;		PRINT	'Ende'
;;		PRINT	<CR,LF,'Hallo'>
;;		PRINT	,12
;;
;; Extern:	PCHAR
;;
	LOCAL AROUND,MESG
	PUSH	H
	PUSH	B
	 IF	NUL BYTES
	LXI	H,MESG
	MVI	B,AROUND-MESG
	 ELSE
	 IF	NOT NUL TEXT
	LXI	H,TEXT
	 ENDIF
	MVI	B,BYTES
	 ENDIF
	CALL	PBUF?
	POP	B
	POP	H
	 IF	NOT PRFLAG OR NUL BYTES
	JMP	AROUND
	 ENDIF
	 IF	NOT PRFLAG
;
PBUF?:	MOV	A,M
	PCHAR
	INX	H
	DCR	B
	JNZ	PBUF?
	RET
;
PRFLAG	SET	TRUE
	 ENDIF
	 IF	NUL BYTES
;
MESG:	DB	TEXT
	 ENDIF
AROUND:
;;
	 ENDM
;
;
;
PROTEC	 MACRO	POINTR
;; 24.08.1986
;; Inline Macro zum schuetzen der Datei
;; in FCB bei POINTR.
;;
;; Extern:	SYSF
;;
	LOCAL AROUND,PROT2?
	LXI	D,POINTR
	LDA	POINTR+9
	ORI	80H
	STA	POINTR+9
	CALL	PROT2?
	JMP	AROUND
;
PROT2?:	SYSF 30
;
AROUND:
;;
	 ENDM
;
;
;
READB	 MACRO	
;; 16.08.1986
;; Inline Macro fuer die Eingabe einer
;; Zeile von der Konsole.
;;
;; Extern:	BDOS
;;
	LOCAL AROUND,RBUFM,RBUF,RBUFC,RBUFE
	CALL	RDB2?
	 IF	NOT RCFLAG
	JMP	AROUND
;
RDB2?:	PUSH	H
	PUSH	D
	PUSH	B
	LXI	D,RBUFM
	MVI	C,10
	CALL	BDOS
	LXI	H,RBUFM+2
	SHLD	RBUFM-2
	POP	B
	POP	D
	POP	H
	RET
;
; Globale Routine
;
GETCH:	LDA	RBUFC
	SUI	1
	RC
	STA	RBUFC
	PUSH	H
	LHLD	RBUFP
	MOV	A,M
	INX	H
	SHLD	RBUFP
	POP	H
;
; Zeichen auf Trennzeichen ueberpruefen
; Z-Flag gesetzt wenn '=_.:;<>'
;
;	CP	'='
;	RET	Z
;	CP	'_'
;	RET	Z
;	CP	'.'
;	RET	Z
;	CP	':'
;	RET	Z
;	CP	';'
;	RET	Z
;	CP	'<'
;	RET	Z
;	CP	'>'
;	RET	Z
;
; kein Trennzeichen
;
	RET
;
RCFLAG	SET	TRUE
;
RBUFP:	DW	RBUF
;
RBUFM:	DB	RBUFE-RBUF
RBUFC:	DS	1,0
RBUF:	DS	16,0
RBUFE:
	 ENDIF
AROUND:
;;
	 ENDM
;
;
;
READCH	 MACRO	REG
;; 16.08.1986
;; Inline Macro zum Lesen eines Zeichens
;; von der Konsole. Das Zeichen wird in A
;; uebergeben wenn der zweite Parameter fehlt.
;;
;; Anwendung:	READCH
;;		READCH C
;;
;; Extern:	SYSF
;;
	LOCAL AROUND
	CALL	RDCH?
	 IF	NOT NUL REG
	# LD REG,A
	 ENDIF
	 IF	NOT CIFLAG
	JMP	AROUND
;
RDCH?:	SYSF 1
CIFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
READS	 MACRO	POINTR,STAR
;; 18.08.1986
;; Inline Macro zum lesen eines
;; Sektors. POINTR zeigt auf Datei-
;; steuerblock. STAR ist das nach
;; dem lesen auszugebende Zeichen.
;; Z-FLAG ist bei Dateiende geloescht.
;;
;; Anwendung:	READS	FCB1
;;		READS	FCBS,'*'
;;
;; Extern:	SYSF, PCHAR
;;
	LOCAL AROUND
	 IF	NOT NUL STAR
	PCHAR STAR
	 ENDIF
	 IF	NOT NUL POINTR
	LXI	D,POINTR
	 ENDIF
	CALL	READ2?
	ORA	A
	 IF	NOT RDFLAG
	JMP	AROUND
;
READ2?:	SYSF 20
;
RDFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
RENAME	 MACRO	POINTR,SHOW
;; 19.08.1986
;; Inline Macro zum umbenennen einer
;; existierenden Datei. POINTR zeigt
;; auf Originalnamen. Neuer Name ist
;; bei POINTR + 10H. SHOW ist ein Dummy-
;; parameter. Wenn gesetzt, werden die
;; Filenamen auf Konsole ausgegeben.
;;
;; Anwendung:	RENAME	FCB1
;;		RENAME	FCB1,*
;;
;; Extern:	SYSF,PRINT,UNPROT,CRLF
;;
	LOCAL AROUND,REN2?
	LXI	D,POINTR
	LDA	POINTR+9
	ORI	80H
	JZ	REN2?
	UNPROT POINTR
;
REN2?:	CALL	RENAM?
	 IF	NOT NUL SHOW
	CRLF
	PRINT POINTR+1,11
	PRINT ' --> '
	PRINT POINTR+11H,11
	 ENDIF
	 IF	NOT RNFLAG
	JMP	AROUND
;
RENAM?:	SYSF 23
;
RNFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
SETDMA	 MACRO	POINTR
;; 18.08.1986
;; Inline Macro zum setzen der
;; DMA-Adresse.
;;
;; Extern:	SYSF
;;
	LOCAL AROUND
	 IF	NOT NUL POINTR
	LXI	D,POINTR
	 ENDIF
	CALL	DMA2?
	 IF	NOT DMFLAG
	JMP	AROUND
;
DMA2?:	SYSF 26
;
DMFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
SETUP2	 MACRO	
;; 19.08.1986
;; Inline Macro zum eroeffnen zweier Dateien.
;; Eingabedatei ist der erste Parameter einer
;; Kommandozeile. Der Dateisteuerblock ist FCB1
;; bei 5C Hex. Die Ausgabedatei ist der zweite
;; Parameter.
;;
;; Extern:	MOVE,OPEN,MAKE,DELETE,ERRORM,AMBIG,COMPAR
;;
	LOCAL AROUND,SET2?,SET3?,SET4?
S2FLAG	SET	TRUE
;
	LDA	FCB2+1
	CPI	BLANK
	JNZ	SET4?
	MOVE FCB1+1,FCB2+1,11
;
SET4?:	AMBIG FCB1,FCB2
	COMPAR FCB1,FCB2,12
	JZ	DUPNM?
;
SET2?:	MOVE FCB2,DFCB,16
	OPEN FCB1
	JC	AROUND
	OPEN DFCB,SET3?
	JC	AROUND
;
SET3?:	DELETE DFCB
	JC	AROUND
	MAKE DFCB
	JMP	AROUND
;
DUPNM?:	MVI	A,TRUE
	STA	DUPL
	MOVE '$$$',FCB2+9
	JMP	SET2?
;
DUPL:	DB	FALSE
;
DFCB:	DS	33,0
;
AROUND:
;;
	 ENDM
;
;
;
SYSF	 MACRO	FUNC,AE
;; 16.08.1986
;; Macro zum generieren von BDOS-Calls.
;; FUNC ist die BDOS-Funktionsnummer in C.
;; Lade A nach E wenn der zweite Parameter
;; angegeben ist.
;;
;; Anwendung:	SYSF	15
;;		SYSF	2,AE
;;
;; Extern:	BDOS
;;
	PUSH	H
	PUSH	D
	PUSH	B
	MVI	C,FUNC
	 IF	NOT NUL AE
	MOV	E,A
	PUSH	PSW
	CALL	BDOS
	POP	PSW
	 ELSE
	CALL	BDOS
	 ENDIF
	POP	B
	POP	D
	POP	H
	RET
;;
	 ENDM
;
;
;
UCASE	 MACRO	REG
;; 16.08.1986
;; Inline Macro um ein Zeichen in einem
;; beliebigen Register in einen Gross-
;; buchstaben umzuwandeln. Ohne Parameter
;; wird Register A angenommen.
;;
;; Anwendung:	UCASE
;;		UCASE C
;;
	LOCAL NOTUP?
	 IF	NOT NUL REG
	PUSH	PSW
	MVI	A,REG
	 ENDIF
	CPI	'Z'+7
	JC	NOTUP?
	ANI	5FH
;
NOTUP?:	 IF	NOT NUL REG
	# LD REG,A
	POP	PSW
	 ENDIF
;;
	 ENDM
;
;
;
UNPROT	 MACRO	POINTR
;; 19.08.1986
;; Inline Macro zum aendern von R/O
;; in R/W. POINTR zeigt auf Datei-
;; steuerblock.
;;
;; Extern:	SYSF
;;
	LOCAL AROUND
	LXI	D,POINTR
	LDA	POINTR+9
	ANI	7FH
	STA	POINTR+9
	CALL	UNPR2?
	 IF	NOT UNFLAG
	JMP	AROUND
;
UNPR2?:	SYSF 30
;
UNFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM
;
;
;
UPPER	 MACRO	REG
;; 16.08.1986
;; Inline Macro laedt die oberen vier Bits
;; des Akkus in die unteren vier Bits und
;; loescht die oberen vier Bits.
;;
;; Anwendung:	UPPER
;;		OUTHEX (drucken)
;;
	 IF	NOT NUL REG
	PUSH	PSW
	MVI	A,REG
	 ENDIF
	RAR
	RAR
	RAR
	RAR
	ANI	0FH
	 IF	NOT NUL REG
	# LD REG,A
	POP	PSW
	 ENDIF
;;
	 ENDM
;
;
;
VERSN	 MACRO	TEXT
;; 16.08.1986
;; Inline Macro zum einbetten der
;; Versionsnummer. NUM wird in
;; Apostrophe eingeschlossen.
;;
;; Anwendung:	VERSN 'COPYSYS 23.08.1986'
;;
	LOCAL AROUND
	JMP	AROUND
VER:
	DB	TEXT,', (c) by AK$'
;
AROUND:
;;
	 ENDM
;
;
;
WAIT	 MACRO	TIME
;; Inline Macro zum erzeugen von
;; einer bestimmten Verzoegerung.
;; 
;; Anwendung:	WAIT 	5
;;
	LOCAL AROUND
	 IF	NOT NUL TIME
	MVI	A,TIME
	 ENDIF
	CALL	WAIT?
	 IF	NOT WAFLAG
	JMP	AROUND
;
WAIT?:	PUSH	PSW
	PUSH	B
	LXI	B,0FFFFH
WAIT1?:	DCX	B
	MOV	A,B
	ORA	C
	JNZ	WAIT1?
	POP	B
	POP	PSW
	DCR	A
	JNZ	WAIT?
	RET
;
WAFLAG	SET	TRUE
	 ENDIF
;;
AROUND:
;
	 ENDM
;
;
;
WRFILE	 MACRO	FCB,POINTR,STAR
;; 03.01.1987
;; Inline Macro zum schreiben einer Datei
;; aus dem Speicher.
;;
;; Extern:	WRITES,SETDMA,BUFFER
;;
	LOCAL WRFIL?,EVEN?,OK?,AROUND

	# LD DE,(POINTR)	; Ende
	LXI	H,BUFFER	; Start
	SHLD	POINTR
	XCHG
	XRA	A
	# SBC HL,DE		; Laenge in Bytes
	MOV	A,L
	MOV	L,H
	MVI	H,0
	DAD	H
	ORA	A
	JZ	EVEN?
	INX	H
EVEN?:	MOV	A,H
	ORA	L
	JNZ	OK?
	GOTOXY 3,22
	CLREOL
	PRINT <BEL,'Buffer ist leer!'>
	STC
	JMP	AROUND
;
OK?:	PUSH	B
	MOV	B,H
	MOV	C,L
;
WRFIL?:	LHLD	POINTR
	XCHG
	SETDMA
	WRITES FCB,STAR
	LHLD	POINTR
	LXI	D,80H
	DAD	D
	SHLD	POINTR
	DCX	B
	MOV	A,C
	ORA	B
	JNZ	WRFIL?
	POP	B
;;
AROUND:
;
	 ENDM
;
;
;
WRITES	 MACRO	POINTR,STAR
;; 19.08.1986
;; Inline Macro zum schreiben eines
;; Sektors. POINTR zeigt auf Datei-
;; Steuerblock. STAR kann wahlweise
;; auf der Konsole augeben werden.
;;
	LOCAL AROUND
	 IF	NOT NUL STAR
	PCHAR STAR
	 ENDIF
	 IF	NOT NUL POINTR
	LXI	D,POINTR
	 ENDIF
	CALL	WRIT2?
	ORA	A
	 IF	WRFLAG
	JNZ	NROOM?
	 ELSE
	JZ	AROUND
;
NROOM?:	GOTOXY 3,22
	CLREOL
	PRINT <BEL,'Diskette ist voll!'>
	STC
	JMP	AROUND
;
WRIT2?:	SYSF 21
;
WRFLAG	SET	TRUE
	 ENDIF
;
AROUND:
;;
	 ENDM

;
;
;
;
; Line numbers containing untranslated opcodes:
;
; 00220 00258 00259 00595 00640 00681 00916 01092 01438 01658
; 01713 01785 01790
;
